import os
from unittest import TestCase

import requests
from lxml import html

from basketball_reference_web_scraper.data import Team, Position, POSITION_ABBREVIATIONS_TO_POSITION, \
    TEAM_ABBREVIATIONS_TO_TEAM
from basketball_reference_web_scraper.html import PlayerSeasonTotalTable
from basketball_reference_web_scraper.parsers import PositionAbbreviationParser, TeamAbbreviationParser, \
    PlayerSeasonTotalsParser


class TestPlayersSeasonTotals(TestCase):
    def setUp(self):
        self.season_2001_totals = requests.get(
            'https://www.basketball-reference.com/leagues/NBA_2001_totals.html'
        ).text
        self.season_2018_totals = requests.get(
            'https://www.basketball-reference.com/leagues/NBA_2018_totals.html'
        ).text
        self.season_2019_totals = requests.get(
            'https://www.basketball-reference.com/leagues/NBA_2019_totals.html'
        ).text
        self.jemerrio_jones_blank_age_2019_totals_file = open(
            os.path.join(
                os.path.dirname(__file__),
                '../files/NBA_2019_totals_jemerrio_jones_blank_age.html',
            )
        )
        self.jemerrio_jones_with_a_blank_age_2019_totals = self.jemerrio_jones_blank_age_2019_totals_file.read()
        self.parser = PlayerSeasonTotalsParser(
            position_abbreviation_parser=PositionAbbreviationParser(
                abbreviations_to_positions=POSITION_ABBREVIATIONS_TO_POSITION
            ),
            team_abbreviation_parser=TeamAbbreviationParser(
                abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM,
            )
        )

    def tearDown(self):
        self.jemerrio_jones_blank_age_2019_totals_file.close()

    def test_2001_players_season_totals(self):
        table = PlayerSeasonTotalTable(html=html.fromstring(self.season_2001_totals))
        parsed_season_totals = self.parser.parse(table.rows)
        self.assertEqual(len(parsed_season_totals), 490)

        mahmoud_abdul_rauf = parsed_season_totals[0]

        self.assertEqual(mahmoud_abdul_rauf["slug"], "abdulma02")
        self.assertEqual(mahmoud_abdul_rauf["name"], "Mahmoud Abdul-Rauf")
        self.assertEqual(mahmoud_abdul_rauf["positions"], [Position.POINT_GUARD])
        self.assertEqual(mahmoud_abdul_rauf["team"], Team.VANCOUVER_GRIZZLIES)
        self.assertEqual(mahmoud_abdul_rauf["games_played"], 41)
        self.assertEqual(mahmoud_abdul_rauf["games_started"], 0)
        self.assertEqual(mahmoud_abdul_rauf["minutes_played"], 486)
        self.assertEqual(mahmoud_abdul_rauf["made_field_goals"], 120)
        self.assertEqual(mahmoud_abdul_rauf["attempted_field_goals"], 246)
        self.assertEqual(mahmoud_abdul_rauf["made_three_point_field_goals"], 4)
        self.assertEqual(mahmoud_abdul_rauf["attempted_three_point_field_goals"], 14)
        self.assertEqual(mahmoud_abdul_rauf["made_free_throws"], 22)
        self.assertEqual(mahmoud_abdul_rauf["attempted_free_throws"], 29)
        self.assertEqual(mahmoud_abdul_rauf["offensive_rebounds"], 5)
        self.assertEqual(mahmoud_abdul_rauf["defensive_rebounds"], 20)
        self.assertEqual(mahmoud_abdul_rauf["assists"], 76)
        self.assertEqual(mahmoud_abdul_rauf["steals"], 9)
        self.assertEqual(mahmoud_abdul_rauf["blocks"], 1)
        self.assertEqual(mahmoud_abdul_rauf["turnovers"], 26)
        self.assertEqual(mahmoud_abdul_rauf["personal_fouls"], 50)

    def test_2018_players_season_totals(self):
        table = PlayerSeasonTotalTable(html=html.fromstring(self.season_2018_totals))
        parsed_season_totals = self.parser.parse(table.rows)
        self.assertEqual(len(parsed_season_totals), 605)

        alex_abrines = parsed_season_totals[0]

        self.assertEqual(alex_abrines["slug"], "abrinal01")
        self.assertEqual(alex_abrines["name"], "Álex Abrines")
        self.assertEqual(alex_abrines["positions"], [Position.SHOOTING_GUARD])
        self.assertEqual(alex_abrines["team"], Team.OKLAHOMA_CITY_THUNDER)
        self.assertEqual(alex_abrines["games_played"], 75)
        self.assertEqual(alex_abrines["games_started"], 8)
        self.assertEqual(alex_abrines["minutes_played"], 1134)
        self.assertEqual(alex_abrines["made_field_goals"], 115)
        self.assertEqual(alex_abrines["attempted_field_goals"], 291)
        self.assertEqual(alex_abrines["made_three_point_field_goals"], 84)
        self.assertEqual(alex_abrines["attempted_three_point_field_goals"], 221)
        self.assertEqual(alex_abrines["made_free_throws"], 39)
        self.assertEqual(alex_abrines["attempted_free_throws"], 46)
        self.assertEqual(alex_abrines["offensive_rebounds"], 26)
        self.assertEqual(alex_abrines["defensive_rebounds"], 88)
        self.assertEqual(alex_abrines["assists"], 28)
        self.assertEqual(alex_abrines["steals"], 38)
        self.assertEqual(alex_abrines["blocks"], 8)
        self.assertEqual(alex_abrines["turnovers"], 25)
        self.assertEqual(alex_abrines["personal_fouls"], 124)

    def test_2018_omer_asik_season_totals(self):
        table = PlayerSeasonTotalTable(html=html.fromstring(self.season_2018_totals))
        parsed_season_totals = self.parser.parse(table.rows)

        pelicans_omer_asik = parsed_season_totals[22]

        self.assertEqual(pelicans_omer_asik["slug"], "asikom01")
        self.assertEqual(pelicans_omer_asik["name"], "Ömer Aşık")
        self.assertEqual(pelicans_omer_asik["positions"], [Position.CENTER])
        self.assertEqual(pelicans_omer_asik["team"], Team.NEW_ORLEANS_PELICANS)
        self.assertEqual(pelicans_omer_asik["games_played"], 14)
        self.assertEqual(pelicans_omer_asik["games_started"], 0)
        self.assertEqual(pelicans_omer_asik["minutes_played"], 121)
        self.assertEqual(pelicans_omer_asik["made_field_goals"], 7)
        self.assertEqual(pelicans_omer_asik["attempted_field_goals"], 16)
        self.assertEqual(pelicans_omer_asik["made_three_point_field_goals"], 0)
        self.assertEqual(pelicans_omer_asik["attempted_three_point_field_goals"], 0)
        self.assertEqual(pelicans_omer_asik["made_free_throws"], 4)
        self.assertEqual(pelicans_omer_asik["attempted_free_throws"], 12)
        self.assertEqual(pelicans_omer_asik["offensive_rebounds"], 7)
        self.assertEqual(pelicans_omer_asik["defensive_rebounds"], 30)
        self.assertEqual(pelicans_omer_asik["assists"], 2)
        self.assertEqual(pelicans_omer_asik["steals"], 1)
        self.assertEqual(pelicans_omer_asik["blocks"], 2)
        self.assertEqual(pelicans_omer_asik["turnovers"], 5)
        self.assertEqual(pelicans_omer_asik["personal_fouls"], 14)

        bulls_omer_asik = parsed_season_totals[23]

        self.assertEqual(pelicans_omer_asik["slug"], "asikom01")
        self.assertEqual(bulls_omer_asik["name"], "Ömer Aşık")
        self.assertEqual(bulls_omer_asik["positions"], [Position.CENTER])
        self.assertEqual(bulls_omer_asik["team"], Team.CHICAGO_BULLS)
        self.assertEqual(bulls_omer_asik["games_played"], 4)
        self.assertEqual(bulls_omer_asik["games_started"], 0)
        self.assertEqual(bulls_omer_asik["minutes_played"], 61)
        self.assertEqual(bulls_omer_asik["made_field_goals"], 2)
        self.assertEqual(bulls_omer_asik["attempted_field_goals"], 6)
        self.assertEqual(bulls_omer_asik["made_three_point_field_goals"], 0)
        self.assertEqual(bulls_omer_asik["attempted_three_point_field_goals"], 0)
        self.assertEqual(bulls_omer_asik["made_free_throws"], 0)
        self.assertEqual(bulls_omer_asik["attempted_free_throws"], 1)
        self.assertEqual(bulls_omer_asik["offensive_rebounds"], 2)
        self.assertEqual(bulls_omer_asik["defensive_rebounds"], 8)
        self.assertEqual(bulls_omer_asik["assists"], 1)
        self.assertEqual(bulls_omer_asik["steals"], 1)
        self.assertEqual(bulls_omer_asik["blocks"], 2)
        self.assertEqual(bulls_omer_asik["turnovers"], 4)
        self.assertEqual(bulls_omer_asik["personal_fouls"], 6)

    def test_2019_jimmy_butler_season_totals(self):
        table = PlayerSeasonTotalTable(html=html.fromstring(self.season_2019_totals))
        parsed_season_totals = self.parser.parse(table.rows)

        philly_jimmy_butler = parsed_season_totals[94]

        self.assertEqual(philly_jimmy_butler["slug"], "butleji01")
        self.assertEqual(philly_jimmy_butler["name"], "Jimmy Butler")
        self.assertEqual(philly_jimmy_butler["positions"], [Position.SMALL_FORWARD])
        self.assertEqual(philly_jimmy_butler["team"], Team.PHILADELPHIA_76ERS)
        self.assertEqual(philly_jimmy_butler["games_played"], 55)
        self.assertEqual(philly_jimmy_butler["games_started"], 55)
        self.assertEqual(philly_jimmy_butler["minutes_played"], 1824)
        self.assertEqual(philly_jimmy_butler["made_field_goals"], 344)
        self.assertEqual(philly_jimmy_butler["attempted_field_goals"], 747)
        self.assertEqual(philly_jimmy_butler["made_three_point_field_goals"], 50)
        self.assertEqual(philly_jimmy_butler["attempted_three_point_field_goals"], 148)
        self.assertEqual(philly_jimmy_butler["made_free_throws"], 264)
        self.assertEqual(philly_jimmy_butler["attempted_free_throws"], 304)
        self.assertEqual(philly_jimmy_butler["offensive_rebounds"], 105)
        self.assertEqual(philly_jimmy_butler["defensive_rebounds"], 185)
        self.assertEqual(philly_jimmy_butler["assists"], 220)
        self.assertEqual(philly_jimmy_butler["steals"], 99)
        self.assertEqual(philly_jimmy_butler["blocks"], 29)
        self.assertEqual(philly_jimmy_butler["turnovers"], 81)
        self.assertEqual(philly_jimmy_butler["personal_fouls"], 93)

    def test_2019_jemerrio_jones_blank_age_season_totals(self):
        table = PlayerSeasonTotalTable(html=html.fromstring(self.jemerrio_jones_with_a_blank_age_2019_totals))
        parsed_season_totals = self.parser.parse(table.rows)

        jemerrio_jones = parsed_season_totals[310]

        self.assertEqual(jemerrio_jones["slug"], "jonesje01")
        self.assertEqual(jemerrio_jones["name"], "Jemerrio Jones")
        self.assertEqual(jemerrio_jones["positions"], [Position.SMALL_FORWARD])
        self.assertIsNone(jemerrio_jones["age"])
        self.assertEqual(jemerrio_jones["team"], Team.LOS_ANGELES_LAKERS)
        self.assertEqual(jemerrio_jones["games_played"], 1)
        self.assertEqual(jemerrio_jones["games_started"], 0)
        self.assertEqual(jemerrio_jones["minutes_played"], 5)
        self.assertEqual(jemerrio_jones["made_field_goals"], 1)
        self.assertEqual(jemerrio_jones["attempted_field_goals"], 2)
        self.assertEqual(jemerrio_jones["made_three_point_field_goals"], 0)
        self.assertEqual(jemerrio_jones["attempted_three_point_field_goals"], 1)
        self.assertEqual(jemerrio_jones["made_free_throws"], 0)
        self.assertEqual(jemerrio_jones["attempted_free_throws"], 0)
        self.assertEqual(jemerrio_jones["offensive_rebounds"], 1)
        self.assertEqual(jemerrio_jones["defensive_rebounds"], 0)
        self.assertEqual(jemerrio_jones["assists"], 0)
        self.assertEqual(jemerrio_jones["steals"], 1)
        self.assertEqual(jemerrio_jones["blocks"], 0)
        self.assertEqual(jemerrio_jones["turnovers"], 0)
        self.assertEqual(jemerrio_jones["personal_fouls"], 0)



from unittest import TestCase
from unittest.mock import MagicMock

from basketball_reference_web_scraper.html import PlayerPageTotalsRow


class TestPlayerPageTotalsRow(TestCase):
    def test_league_abbreviation_is_none_when_no_matching_league_abbreviations(self):
        html = MagicMock()
        html.xpath = MagicMock(return_value=[])

        self.assertIsNone(PlayerPageTotalsRow(html=html).league_abbreviation)
        html.xpath.assert_called_once_with('.//td[@data-stat="lg_id"]')

    def test_league_abbreviation_is_first_abbreviation_text_content_when_matching_league_abbreviations(self):
        first_abbreviation = MagicMock()
        first_abbreviation.text_content = MagicMock(return_value="first abbreviation")

        second_abbreviation = MagicMock()
        second_abbreviation.text_content = MagicMock(return_value="second abbreviation")

        html = MagicMock()
        html.xpath = MagicMock(return_value=[first_abbreviation, second_abbreviation])

        self.assertEqual(
            PlayerPageTotalsRow(html=html).league_abbreviation,
            "first abbreviation"
        )
        html.xpath.assert_called_once_with('.//td[@data-stat="lg_id"]')

    def test_different_class_is_not_equal(self):
        self.assertNotEqual(
            PlayerPageTotalsRow(html=MagicMock()),
            "jaebaebae"
        )

    def test_different_html_but_same_class_is_not_equal(self):
        self.assertNotEqual(
            PlayerPageTotalsRow(html=MagicMock()),
            PlayerPageTotalsRow(html=MagicMock())
        )

    def test_same_html_and_same_class_is_equal(self):
        html = MagicMock()
        self.assertEqual(
            PlayerPageTotalsRow(html=html),
            PlayerPageTotalsRow(html=html),
        )





import requests
from lxml import html

from basketball_reference_web_scraper.data import TEAM_TO_TEAM_ABBREVIATION, TEAM_ABBREVIATIONS_TO_TEAM, TeamTotal, \
    LOCATION_ABBREVIATIONS_TO_POSITION, OUTCOME_ABBREVIATIONS_TO_OUTCOME, TEAM_NAME_TO_TEAM, \
    POSITION_ABBREVIATIONS_TO_POSITION, LEAGUE_ABBREVIATIONS_TO_LEAGUE, PlayerData
from basketball_reference_web_scraper.errors import InvalidDate, InvalidPlayerAndSeason
from basketball_reference_web_scraper.html import PlayerSeasonTotalTable, BoxScoresPage, DailyLeadersPage, \
    PlayerAdvancedSeasonTotalsTable, PlayByPlayPage, DailyBoxScoresPage, SchedulePage, PlayerSeasonBoxScoresPage, \
    SearchPage, PlayerPage
from basketball_reference_web_scraper.parsers import PositionAbbreviationParser, TeamAbbreviationParser, \
    PlayerSeasonTotalsParser, TeamTotalsParser, LocationAbbreviationParser, OutcomeAbbreviationParser, \
    SecondsPlayedParser, PlayerBoxScoresParser, PlayerAdvancedSeasonTotalsParser, PeriodDetailsParser, \
    PeriodTimestampParser, ScoresParser, PlayByPlaysParser, TeamNameParser, ScheduledStartTimeParser, \
    ScheduledGamesParser, PlayerBoxScoreOutcomeParser, PlayerSeasonBoxScoresParser, SearchResultNameParser, \
    ResourceLocationParser, SearchResultsParser, LeagueAbbreviationParser, PlayerDataParser

BASE_URL = 'https://www.basketball-reference.com'
PLAY_BY_PLAY_TIMESTAMP_FORMAT = "%M:%S.%f"
PLAY_BY_PLAY_SCORES_REGEX = "(?P<away_team_score>[0-9]+)-(?P<home_team_score>[0-9]+)"
SEARCH_RESULT_RESOURCE_LOCATION_REGEX = '(https?:\/\/www\.basketball-reference\.com\/)?(?P<resource_type>.+?(?=\/)).*\/(?P<resource_identifier>.+).html'


def player_box_scores(day, month, year):
    url = '{BASE_URL}/friv/dailyleaders.cgi?month={month}&day={day}&year={year}'.format(
        BASE_URL=BASE_URL,
        day=day,
        month=month,
        year=year
    )

    response = requests.get(url=url, allow_redirects=False)

    response.raise_for_status()

    if response.status_code == requests.codes.ok:
        page = DailyLeadersPage(html=html.fromstring(response.content))
        box_score_parser = PlayerBoxScoresParser(
            team_abbreviation_parser=TeamAbbreviationParser(
                abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM
            ),
            location_abbreviation_parser=LocationAbbreviationParser(
                abbreviations_to_locations=LOCATION_ABBREVIATIONS_TO_POSITION
            ),
            outcome_abbreviation_parser=OutcomeAbbreviationParser(
                abbreviations_to_outcomes=OUTCOME_ABBREVIATIONS_TO_OUTCOME
            ),
            seconds_played_parser=SecondsPlayedParser(),
        )
        return box_score_parser.parse(page.daily_leaders)

    raise InvalidDate(day=day, month=month, year=year)


def regular_season_player_box_scores(player_identifier, season_end_year):
    # Makes assumption that basketball reference pattern of breaking out player pathing using first character of
    # surname can be derived from the fact that basketball reference also has a pattern of player identifiers
    # starting with first few characters of player's surname
    url = '{BASE_URL}/players/{player_surname_starting_character}/{player_identifier}/gamelog/{season_end_year}'.format(
        BASE_URL=BASE_URL,
        player_surname_starting_character=player_identifier[0],
        player_identifier=player_identifier,
        season_end_year=season_end_year,
    )

    response = requests.get(url=url, allow_redirects=False)
    response.raise_for_status()

    page = PlayerSeasonBoxScoresPage(html=html.fromstring(response.content))
    if page.regular_season_box_scores_table is None:
        raise InvalidPlayerAndSeason(player_identifier=player_identifier, season_end_year=season_end_year)

    parser = PlayerSeasonBoxScoresParser(
        team_abbreviation_parser=TeamAbbreviationParser(
            abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM
        ),
        location_abbreviation_parser=LocationAbbreviationParser(
            abbreviations_to_locations=LOCATION_ABBREVIATIONS_TO_POSITION
        ),
        outcome_parser=PlayerBoxScoreOutcomeParser(
            outcome_abbreviation_parser=OutcomeAbbreviationParser(
                abbreviations_to_outcomes=OUTCOME_ABBREVIATIONS_TO_OUTCOME
            ),
        ),
        seconds_played_parser=SecondsPlayedParser(),
    )
    return parser.parse(box_scores=page.regular_season_box_scores_table.rows)


def schedule_for_month(url):
    response = requests.get(url=url)

    response.raise_for_status()

    page = SchedulePage(html=html.fromstring(html=response.content))
    parser = ScheduledGamesParser(
        start_time_parser=ScheduledStartTimeParser(),
        team_name_parser=TeamNameParser(team_names_to_teams=TEAM_NAME_TO_TEAM),
    )
    return parser.parse_games(games=page.rows)


def season_schedule(season_end_year):
    url = '{BASE_URL}/leagues/NBA_{season_end_year}_games.html'.format(
        BASE_URL=BASE_URL,
        season_end_year=season_end_year
    )

    response = requests.get(url=url)

    response.raise_for_status()

    page = SchedulePage(html=html.fromstring(html=response.content))
    parser = ScheduledGamesParser(
        start_time_parser=ScheduledStartTimeParser(),
        team_name_parser=TeamNameParser(team_names_to_teams=TEAM_NAME_TO_TEAM),
    )
    season_schedule_values = parser.parse_games(games=page.rows)

    for month_url_path in page.other_months_schedule_urls:
        url = '{BASE_URL}{month_url_path}'.format(BASE_URL=BASE_URL, month_url_path=month_url_path)
        monthly_schedule = schedule_for_month(url=url)
        season_schedule_values.extend(monthly_schedule)

    return season_schedule_values


def players_season_totals(season_end_year):
    url = '{BASE_URL}/leagues/NBA_{season_end_year}_totals.html'.format(
        BASE_URL=BASE_URL,
        season_end_year=season_end_year,
    )

    response = requests.get(url=url)

    response.raise_for_status()

    table = PlayerSeasonTotalTable(html=html.fromstring(response.content))
    parser = PlayerSeasonTotalsParser(
        position_abbreviation_parser=PositionAbbreviationParser(
            abbreviations_to_positions=POSITION_ABBREVIATIONS_TO_POSITION
        ),
        team_abbreviation_parser=TeamAbbreviationParser(
            abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM,
        )
    )
    return parser.parse(table.rows)


def players_advanced_season_totals(season_end_year, include_combined_values=False):
    url = '{BASE_URL}/leagues/NBA_{season_end_year}_advanced.html'.format(
        BASE_URL=BASE_URL,
        season_end_year=season_end_year,
    )

    response = requests.get(url=url)

    response.raise_for_status()

    table = PlayerAdvancedSeasonTotalsTable(html=html.fromstring(response.content))
    parser = PlayerAdvancedSeasonTotalsParser(
        team_abbreviation_parser=TeamAbbreviationParser(
            abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM
        ),
        position_abbreviation_parser=PositionAbbreviationParser(
            abbreviations_to_positions=POSITION_ABBREVIATIONS_TO_POSITION
        )
    )

    return parser.parse(table.get_rows(include_combined_values))


def team_box_score(game_url_path):
    url = "{BASE_URL}/{game_url_path}".format(BASE_URL=BASE_URL, game_url_path=game_url_path)

    response = requests.get(url=url)

    response.raise_for_status()

    page = BoxScoresPage(html.fromstring(response.content))
    combined_team_totals = [
        TeamTotal(team_abbreviation=table.team_abbreviation, totals=table.team_totals)
        for table in page.basic_statistics_tables
    ]
    parser = TeamTotalsParser(team_abbreviation_parser=TeamAbbreviationParser(
        abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM,
    ))

    return parser.parse(
        first_team_totals=combined_team_totals[0],
        second_team_totals=combined_team_totals[1],
    )


def team_box_scores(day, month, year):
    url = "{BASE_URL}/boxscores/".format(BASE_URL=BASE_URL)

    response = requests.get(url=url, params={"day": day, "month": month, "year": year})

    response.raise_for_status()

    page = DailyBoxScoresPage(html=html.fromstring(response.content))

    return [
        box_score
        for game_url_path in page.game_url_paths
        for box_score in team_box_score(game_url_path=game_url_path)
    ]


def play_by_play(home_team, day, month, year):
    add_0_if_needed = lambda s: "0" + s if len(s) == 1 else s

    # the hard-coded `0` in the url assumes we always take the first match of the given date and team.
    url = "{BASE_URL}/boxscores/pbp/{year}{month}{day}0{team_abbr}.html".format(
        BASE_URL=BASE_URL, year=year, month=add_0_if_needed(str(month)), day=add_0_if_needed(str(day)),
        team_abbr=TEAM_TO_TEAM_ABBREVIATION[home_team]
    )
    response = requests.get(url=url)
    response.raise_for_status()
    page = PlayByPlayPage(html=html.fromstring(response.content))

    play_by_plays_parser = PlayByPlaysParser(
        period_details_parser=PeriodDetailsParser(regulation_periods_count=4),
        period_timestamp_parser=PeriodTimestampParser(timestamp_format=PLAY_BY_PLAY_TIMESTAMP_FORMAT),
        scores_parser=ScoresParser(scores_regex=PLAY_BY_PLAY_SCORES_REGEX))

    team_name_parser = TeamNameParser(team_names_to_teams=TEAM_NAME_TO_TEAM)

    return play_by_plays_parser.parse(play_by_plays=page.play_by_play_table.rows,
                                      away_team=team_name_parser.parse_team_name(team_name=page.away_team_name),
                                      home_team=team_name_parser.parse_team_name(team_name=page.home_team_name))


def search(term):
    response = requests.get(
        url="{BASE_URL}/search/search.fcgi".format(BASE_URL=BASE_URL),
        params={"search": term}
    )

    response.raise_for_status()

    player_results = []

    if response.url.startswith("{BASE_URL}/search/search.fcgi".format(BASE_URL=BASE_URL)):
        page = SearchPage(html=html.fromstring(response.content))

        parser = SearchResultsParser(
            search_result_name_parser=SearchResultNameParser(),
            search_result_location_parser=ResourceLocationParser(
                resource_location_regex=SEARCH_RESULT_RESOURCE_LOCATION_REGEX,
            ),
            league_abbreviation_parser=LeagueAbbreviationParser(
                abbreviations_to_league=LEAGUE_ABBREVIATIONS_TO_LEAGUE,
            ),
        )

        parsed_results = parser.parse(nba_aba_baa_players=page.nba_aba_baa_players)
        player_results += parsed_results["players"]

        while page.nba_aba_baa_players_pagination_url is not None:
            response = requests.get(
                url="{BASE_URL}/search/{pagination_url}".format(
                    BASE_URL=BASE_URL,
                    pagination_url=page.nba_aba_baa_players_pagination_url
                )
            )

            response.raise_for_status()

            page = SearchPage(html=html.fromstring(response.content))

            parsed_results = parser.parse(nba_aba_baa_players=page.nba_aba_baa_players)
            player_results += parsed_results["players"]

    elif response.url.startswith("{BASE_URL}/players".format(BASE_URL=BASE_URL)):
        page = PlayerPage(html=html.fromstring(response.content))
        data = PlayerData(
            name=page.name,
            resource_location=response.url,
            league_abbreviations=set([row.league_abbreviation for row in page.totals_table.rows])
        )
        parser = PlayerDataParser(
            search_result_location_parser=ResourceLocationParser(
                resource_location_regex=SEARCH_RESULT_RESOURCE_LOCATION_REGEX,
            ),
            league_abbreviation_parser=LeagueAbbreviationParser(abbreviations_to_league=LEAGUE_ABBREVIATIONS_TO_LEAGUE),
        )
        player_results += [parser.parse(player=data)]

    return {
        "players": player_results
    }



from unittest import TestCase

import requests
from lxml import html

from basketball_reference_web_scraper.data import Team, Position, TEAM_ABBREVIATIONS_TO_TEAM, \
    POSITION_ABBREVIATIONS_TO_POSITION
from basketball_reference_web_scraper.html import PlayerAdvancedSeasonTotalsTable
from basketball_reference_web_scraper.parsers import PlayerAdvancedSeasonTotalsParser, PositionAbbreviationParser, \
    TeamAbbreviationParser


class TestPlayersAdvancedSeasonTotals(TestCase):
    def setUp(self):
        self.season_2019_totals = requests.get(
            'https://www.basketball-reference.com/leagues/NBA_2019_advanced.html'
        ).text
        self.parser = PlayerAdvancedSeasonTotalsParser(
            position_abbreviation_parser=PositionAbbreviationParser(
                abbreviations_to_positions=POSITION_ABBREVIATIONS_TO_POSITION
            ),
            team_abbreviation_parser=TeamAbbreviationParser(
                abbreviations_to_teams=TEAM_ABBREVIATIONS_TO_TEAM,
            ),
        )
        self.season_2019_totals_table = PlayerAdvancedSeasonTotalsTable(html=html.fromstring(self.season_2019_totals))

    def test_2019_jimmy_butler_philly_season_totals(self):
        parsed_season_totals = self.parser.parse(totals=self.season_2019_totals_table.get_rows())

        philly_jimmy_butler = parsed_season_totals[94]

        self.assertEqual(philly_jimmy_butler["slug"], "butleji01")
        self.assertEqual(philly_jimmy_butler["name"], "Jimmy Butler")
        self.assertEqual(philly_jimmy_butler["positions"], [Position.SMALL_FORWARD])
        self.assertEqual(philly_jimmy_butler["team"], Team.PHILADELPHIA_76ERS)
        self.assertEqual(philly_jimmy_butler["games_played"], 55)
        self.assertEqual(philly_jimmy_butler["minutes_played"], 1824)
        self.assertEqual(philly_jimmy_butler["player_efficiency_rating"], 19.8)
        self.assertEqual(philly_jimmy_butler["true_shooting_percentage"], 0.569)
        self.assertEqual(philly_jimmy_butler["three_point_attempt_rate"], 0.198)
        self.assertEqual(philly_jimmy_butler["free_throw_attempt_rate"], 0.407)
        self.assertEqual(philly_jimmy_butler["offensive_rebound_percentage"], 6.3)
        self.assertEqual(philly_jimmy_butler["defensive_rebound_percentage"], 10.4)
        self.assertEqual(philly_jimmy_butler["total_rebound_percentage"], 8.4)
        self.assertEqual(philly_jimmy_butler["assist_percentage"], 18.0)
        self.assertEqual(philly_jimmy_butler["steal_percentage"], 2.6)
        self.assertEqual(philly_jimmy_butler["block_percentage"], 1.2)
        self.assertEqual(philly_jimmy_butler["turnover_percentage"], 8.4)
        self.assertEqual(philly_jimmy_butler["usage_percentage"], 22.1)
        self.assertEqual(philly_jimmy_butler["offensive_win_shares"], 4.4)
        self.assertEqual(philly_jimmy_butler["defensive_win_shares"], 2.1)
        self.assertEqual(philly_jimmy_butler["win_shares"], 6.6)
        self.assertEqual(philly_jimmy_butler["win_shares_per_48_minutes"], 0.172)
        self.assertEqual(philly_jimmy_butler["offensive_box_plus_minus"], 2.4)
        self.assertEqual(philly_jimmy_butler["defensive_box_plus_minus"], 0.7)
        self.assertEqual(philly_jimmy_butler["box_plus_minus"], 3.1)
        self.assertEqual(philly_jimmy_butler["value_over_replacement_player"], 2.4)

    def test_2019_jimmy_butler_combined_season_totals(self):
        parsed_season_totals = self.parser.parse(totals=self.season_2019_totals_table.get_rows(include_combined_totals=True))

        philly_jimmy_butler = parsed_season_totals[102]

        self.assertEqual(philly_jimmy_butler["slug"], "butleji01")
        self.assertEqual(philly_jimmy_butler["name"], "Jimmy Butler")
        self.assertEqual(philly_jimmy_butler["positions"], [Position.SMALL_FORWARD, Position.SHOOTING_GUARD])
        self.assertEqual(philly_jimmy_butler["team"], None)
        self.assertEqual(philly_jimmy_butler["games_played"], 65)
        self.assertEqual(philly_jimmy_butler["minutes_played"], 2185)
        self.assertEqual(philly_jimmy_butler["player_efficiency_rating"], 20.2)
        self.assertEqual(philly_jimmy_butler["true_shooting_percentage"], 0.571)
        self.assertEqual(philly_jimmy_butler["three_point_attempt_rate"], 0.213)
        self.assertEqual(philly_jimmy_butler["free_throw_attempt_rate"], 0.404)
        self.assertEqual(philly_jimmy_butler["offensive_rebound_percentage"], 6.0)
        self.assertEqual(philly_jimmy_butler["defensive_rebound_percentage"], 10.5)
        self.assertEqual(philly_jimmy_butler["total_rebound_percentage"], 8.3)
        self.assertEqual(philly_jimmy_butler["assist_percentage"], 18.0)
        self.assertEqual(philly_jimmy_butler["steal_percentage"], 2.7)
        self.assertEqual(philly_jimmy_butler["block_percentage"], 1.4)
        self.assertEqual(philly_jimmy_butler["turnover_percentage"], 8.2)
        self.assertEqual(philly_jimmy_butler["usage_percentage"], 22.3)
        self.assertEqual(philly_jimmy_butler["offensive_win_shares"], 5.4)
        self.assertEqual(philly_jimmy_butler["defensive_win_shares"], 2.5)
        self.assertEqual(philly_jimmy_butler["win_shares"], 7.9)
        self.assertEqual(philly_jimmy_butler["win_shares_per_48_minutes"], 0.173)
        self.assertEqual(philly_jimmy_butler["offensive_box_plus_minus"], 2.8)
        self.assertEqual(philly_jimmy_butler["defensive_box_plus_minus"], 0.9)
        self.assertEqual(philly_jimmy_butler["box_plus_minus"], 3.7)
        self.assertEqual(philly_jimmy_butler["value_over_replacement_player"], 3.2)



from unittest import TestCase
from unittest.mock import MagicMock

from basketball_reference_web_scraper.html import PlayerPage, PlayerPageTotalsTable


class TestPlayerPage(TestCase):
    def test_name_is_none_when_no_name_headers(self):
        html = MagicMock()
        html.xpath = MagicMock(return_value=[])

        self.assertIsNone(PlayerPage(html=html).name)
        html.xpath.assert_called_once_with('.//h1[@itemprop="name"]')

    def test_name_is_first_name_header_content_when_name_headers(self):
        first_name_header = MagicMock()
        first_name_header.text_content = MagicMock(return_value="first name")

        second_name_header = MagicMock()
        second_name_header.text_content = MagicMock(return_value="second name")

        html = MagicMock()
        html.xpath = MagicMock(return_value=[first_name_header, second_name_header])

        self.assertEqual(
            PlayerPage(html=html).name,
            "first name"
        )
        html.xpath.assert_called_once_with('.//h1[@itemprop="name"]')

    def test_totals_table_is_none_when_no_totals_tables(self):
        html = MagicMock()
        html.xpath = MagicMock(return_value=[])

        self.assertIsNone(PlayerPage(html=html).totals_table)
        html.xpath.assert_called_once_with('.//table[@id="per_game"]')

    def test_totals_table_is_first_table_when_totals_tables(self):
        first_table = MagicMock()
        second_table = MagicMock()

        html = MagicMock()
        html.xpath = MagicMock(return_value=[first_table, second_table])

        self.assertEqual(
            PlayerPage(html=html).totals_table,
            PlayerPageTotalsTable(html=first_table),
        )
        html.xpath.assert_called_once_with('.//table[@id="per_game"]')



from unittest import TestCase
from unittest.mock import MagicMock

from basketball_reference_web_scraper.data import LEAGUE_ABBREVIATIONS_TO_LEAGUE, League
from basketball_reference_web_scraper.http_client import SEARCH_RESULT_RESOURCE_LOCATION_REGEX
from basketball_reference_web_scraper.parsers import SearchResultsParser, SearchResultNameParser, \
    ResourceLocationParser, LeagueAbbreviationParser


class TestSearchResultsParser(TestCase):
    def setUp(self):
        self.parser = SearchResultsParser(
            search_result_name_parser=SearchResultNameParser(),
            search_result_location_parser=ResourceLocationParser(
                resource_location_regex=SEARCH_RESULT_RESOURCE_LOCATION_REGEX,
            ),
            league_abbreviation_parser=LeagueAbbreviationParser(
                abbreviations_to_league=LEAGUE_ABBREVIATIONS_TO_LEAGUE,
            ),
        )

    def test_parse_single_nba_player(self):
        players = [
            MagicMock(
                resource_name="jaebaebae",
                resource_location="https://www.basketball-reference.com/players/j/jaebaebae.html",
                league_abbreviations="NBA"
            )
        ]
        self.assertEqual(
            self.parser.parse(nba_aba_baa_players=players),
            {
                "players": [
                    {
                        "name": "jaebaebae",
                        "identifier": "jaebaebae",
                        "leagues": {League.NATIONAL_BASKETBALL_ASSOCIATION},
                    }
                ]
            }
        )

    def test_parse_single_aba_player(self):
        players = [
            MagicMock(
                resource_name="jaebaebae",
                resource_location="https://www.basketball-reference.com/players/j/jaebaebae.html",
                league_abbreviations="ABA"
            )
        ]
        self.assertEqual(
            self.parser.parse(nba_aba_baa_players=players),
            {
                "players": [
                    {
                        "name": "jaebaebae",
                        "identifier": "jaebaebae",
                        "leagues": {League.AMERICAN_BASKETBALL_ASSOCIATION},
                    }
                ]
            }
        )

    def test_parse_single_baa_player(self):
        players = [
            MagicMock(
                resource_name="jaebaebae",
                resource_location="https://www.basketball-reference.com/players/j/jaebaebae.html",
                league_abbreviations="BAA"
            )
        ]
        self.assertEqual(
            self.parser.parse(nba_aba_baa_players=players),
            {
                "players": [
                    {
                        "name": "jaebaebae",
                        "identifier": "jaebaebae",
                        "leagues": {League.BASKETBALL_ASSOCIATION_OF_AMERICA},
                    }
                ]
            }
        )

    def test_parse_single_nba_aba_baa_player(self):
        players = [
            MagicMock(
                resource_name="jaebaebae",
                resource_location="https://www.basketball-reference.com/players/j/jaebaebae.html",
                league_abbreviations="NBA/ABA/BAA"
            )
        ]
        self.assertEqual(
            self.parser.parse(nba_aba_baa_players=players),
            {
                "players": [
                    {
                        "name": "jaebaebae",
                        "identifier": "jaebaebae",
                        "leagues": {
                            League.NATIONAL_BASKETBALL_ASSOCIATION,
                            League.AMERICAN_BASKETBALL_ASSOCIATION,
                            League.BASKETBALL_ASSOCIATION_OF_AMERICA
                        },
                    }
                ]
            }
        )

    def test_parse_multiple_nba_aba_baa_players(self):
        players = [
            MagicMock(
                resource_name="jaebaebae1",
                resource_location="https://www.basketball-reference.com/players/j/jaebaebae1.html",
                league_abbreviations="NBA/ABA/BAA"
            ),
            MagicMock(
                resource_name="jaebaebae2",
                resource_location="https://www.basketball-reference.com/players/j/jaebaebae2.html",
                league_abbreviations="NBA/ABA/BAA"
            ),
            MagicMock(
                resource_name="jaebaebae3",
                resource_location="https://www.basketball-reference.com/players/j/jaebaebae3.html",
                league_abbreviations="NBA/ABA/BAA"
            )
        ]
        self.assertEqual(
            self.parser.parse(nba_aba_baa_players=players),
            {
                "players": [
                    {
                        "name": "jaebaebae1",
                        "identifier": "jaebaebae1",
                        "leagues": {
                            League.NATIONAL_BASKETBALL_ASSOCIATION,
                            League.AMERICAN_BASKETBALL_ASSOCIATION,
                            League.BASKETBALL_ASSOCIATION_OF_AMERICA,
                        },
                    },
                    {
                        "name": "jaebaebae2",
                        "identifier": "jaebaebae2",
                        "leagues": {
                            League.NATIONAL_BASKETBALL_ASSOCIATION,
                            League.AMERICAN_BASKETBALL_ASSOCIATION,
                            League.BASKETBALL_ASSOCIATION_OF_AMERICA,
                        },
                    },
                    {
                        "name": "jaebaebae3",
                        "identifier": "jaebaebae3",
                        "leagues": {
                            League.NATIONAL_BASKETBALL_ASSOCIATION,
                            League.AMERICAN_BASKETBALL_ASSOCIATION,
                            League.BASKETBALL_ASSOCIATION_OF_AMERICA,
                        },
                    },
                ]
            }
        )



from unittest import TestCase

import requests
from lxml import html

from basketball_reference_web_scraper.html import DailyBoxScoresPage


class TestDailyBoxScoresPage(TestCase):
    def setUp(self):
        response = requests.get('https://www.basketball-reference.com/boxscores/index.fcgi?month=01&day=01&year=2017')
        self.january_01_2017_box_scores = response.text

    def test_game_url_paths_query(self):
        page = DailyBoxScoresPage(html=html.fromstring(self.january_01_2017_box_scores))
        self.assertEqual(page.game_url_paths_query, '//td[contains(@class, "gamelink")]/a')

    def test_parse_urls(self):
        page = DailyBoxScoresPage(html=html.fromstring(self.january_01_2017_box_scores))
        urls = page.game_url_paths
        self.assertEqual(len(urls), 5)
        self.assertEqual(urls[0], '/boxscores/201701010ATL.html')
        self.assertEqual(urls[1], '/boxscores/201701010IND.html')
        self.assertEqual(urls[2], '/boxscores/201701010LAL.html')
        self.assertEqual(urls[3], '/boxscores/201701010MIA.html')
        self.assertEqual(urls[4], '/boxscores/201701010MIN.html')



from unittest import TestCase

from basketball_reference_web_scraper.data import Team, Location, Outcome, Position
from basketball_reference_web_scraper.writers import RowFormatter


class TestRowFormatter(TestCase):
    def test_away_team_data_field_name(self):
        formatter = RowFormatter(data_field_names=["away_team"])
        self.assertEqual(
            formatter.format(row_data={"away_team": Team.BOSTON_CELTICS}),
            {"away_team": Team.BOSTON_CELTICS.value}
        )

    def test_home_team_data_field_name(self):
        formatter = RowFormatter(data_field_names=["home_team"])
        self.assertEqual(
            formatter.format(row_data={"home_team": Team.BOSTON_CELTICS}),
            {"home_team": Team.BOSTON_CELTICS.value}
        )

    def test_team_data_field_name(self):
        formatter = RowFormatter(data_field_names=["team"])
        self.assertEqual(
            formatter.format(row_data={"team": Team.BOSTON_CELTICS}),
            {"team": Team.BOSTON_CELTICS.value}
        )

    def test_location_data_field_name(self):
        formatter = RowFormatter(data_field_names=["location"])
        self.assertEqual(
            formatter.format(row_data={"location": Location.HOME}),
            {"location": Location.HOME.value}
        )

    def test_opponent_data_field_name(self):
        formatter = RowFormatter(data_field_names=["opponent"])
        self.assertEqual(
            formatter.format(row_data={"opponent": Team.BOSTON_CELTICS}),
            {"opponent": Team.BOSTON_CELTICS.value}
        )

    def test_outcome_data_field_name(self):
        formatter = RowFormatter(data_field_names=["outcome"])
        self.assertEqual(
            formatter.format(row_data={"outcome": Outcome.WIN}),
            {"outcome": Outcome.WIN.value}
        )

    def test_empty_positions_data_field_name(self):
        formatter = RowFormatter(data_field_names=["positions"])
        self.assertEqual(
            formatter.format(row_data={"positions": []}),
            {"positions": ""}
        )

    def test_single_position_data_field_name(self):
        formatter = RowFormatter(data_field_names=["positions"])
        self.assertEqual(
            formatter.format(row_data={"positions": [Position.POINT_GUARD]}),
            {"positions": "POINT GUARD"}
        )

    def test_multiple_positions_data_field_name(self):
        formatter = RowFormatter(data_field_names=["positions"])
        self.assertEqual(
            formatter.format(row_data={"positions": [Position.POINT_GUARD, Position.SHOOTING_GUARD]}),
            {"positions": "POINT GUARD-SHOOTING GUARD"}
        )

    def test_non_team_outcome_location_position_data_field_name(self):
        formatter = RowFormatter(data_field_names=["name"])
        self.assertEqual(
            formatter.format(row_data={"name": "jaebaebae"}),
            {"name": "jaebaebae"}
        )



import json
import os
from pathlib import Path
from unittest import TestCase

import basketball_reference_web_scraper.client as client
from basketball_reference_web_scraper.data import OutputWriteOption, OutputType, Team, Outcome


class TestTeamBoxScoresInMemoryOutput(TestCase):
    def test_2018_01_01_team_box_scores(self):
        team_box_scores = client.team_box_scores(day=1, month=1, year=2018)
        self.assertListEqual(
            team_box_scores,
            [
                {
                    "team": Team.ORLANDO_MAGIC,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 240,
                    "made_field_goals": 35,
                    "attempted_field_goals": 96,
                    "made_three_point_field_goals": 6,
                    "attempted_three_point_field_goals": 31,
                    "made_free_throws": 19,
                    "attempted_free_throws": 25,
                    "offensive_rebounds": 19,
                    "defensive_rebounds": 33,
                    "assists": 16,
                    "steals": 5,
                    "blocks": 5,
                    "turnovers": 12,
                    "personal_fouls": 18,
                    "points": 95,
                },
                {
                    "team": Team.BROOKLYN_NETS,
                    "outcome": Outcome.WIN,
                    "minutes_played": 240,
                    "made_field_goals": 36,
                    "attempted_field_goals": 82,
                    "made_three_point_field_goals": 9,
                    "attempted_three_point_field_goals": 25,
                    "made_free_throws": 17,
                    "attempted_free_throws": 23,
                    "offensive_rebounds": 10,
                    "defensive_rebounds": 42,
                    "assists": 17,
                    "steals": 7,
                    "blocks": 10,
                    "turnovers": 13,
                    "personal_fouls": 22,
                    "points": 98,
                },
                {
                    "team": Team.PORTLAND_TRAIL_BLAZERS,
                    "outcome": Outcome.WIN,
                    "minutes_played": 265,
                    "made_field_goals": 47,
                    "attempted_field_goals": 108,
                    "made_three_point_field_goals": 14,
                    "attempted_three_point_field_goals": 33,
                    "made_free_throws": 16,
                    "attempted_free_throws": 18,
                    "offensive_rebounds": 12,
                    "defensive_rebounds": 38,
                    "assists": 25,
                    "steals": 7,
                    "blocks": 5,
                    "turnovers": 8,
                    "personal_fouls": 16,
                    "points": 124,
                },
                {
                    "team": Team.CHICAGO_BULLS,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 265,
                    "made_field_goals": 45,
                    "attempted_field_goals": 104,
                    "made_three_point_field_goals": 11,
                    "attempted_three_point_field_goals": 27,
                    "made_free_throws": 19,
                    "attempted_free_throws": 23,
                    "offensive_rebounds": 15,
                    "defensive_rebounds": 43,
                    "assists": 28,
                    "steals": 5,
                    "blocks": 7,
                    "turnovers": 12,
                    "personal_fouls": 21,
                    "points": 120,
                },
                {
                    "team": Team.LOS_ANGELES_LAKERS,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 240,
                    "made_field_goals": 34,
                    "attempted_field_goals": 76,
                    "made_three_point_field_goals": 8,
                    "attempted_three_point_field_goals": 26,
                    "made_free_throws": 20,
                    "attempted_free_throws": 29,
                    "offensive_rebounds": 9,
                    "defensive_rebounds": 28,
                    "assists": 18,
                    "steals": 12,
                    "blocks": 4,
                    "turnovers": 24,
                    "personal_fouls": 25,
                    "points": 96,
                },
                {
                    "team": Team.MINNESOTA_TIMBERWOLVES,
                    "outcome": Outcome.WIN,
                    "minutes_played": 240,
                    "made_field_goals": 40,
                    "attempted_field_goals": 81,
                    "made_three_point_field_goals": 7,
                    "attempted_three_point_field_goals": 20,
                    "made_free_throws": 27,
                    "attempted_free_throws": 30,
                    "offensive_rebounds": 7,
                    "defensive_rebounds": 34,
                    "assists": 26,
                    "steals": 14,
                    "blocks": 5,
                    "turnovers": 17,
                    "personal_fouls": 24,
                    "points": 114,
                },
                {
                    "team": Team.MILWAUKEE_BUCKS,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 265,
                    "made_field_goals": 44,
                    "attempted_field_goals": 93,
                    "made_three_point_field_goals": 13,
                    "attempted_three_point_field_goals": 27,
                    "made_free_throws": 26,
                    "attempted_free_throws": 28,
                    "offensive_rebounds": 10,
                    "defensive_rebounds": 33,
                    "assists": 27,
                    "steals": 9,
                    "blocks": 3,
                    "turnovers": 14,
                    "personal_fouls": 23,
                    "points": 127,
                },
                {
                    "team": Team.TORONTO_RAPTORS,
                    "outcome": Outcome.WIN,
                    "minutes_played": 265,
                    "made_field_goals": 45,
                    "attempted_field_goals": 91,
                    "made_three_point_field_goals": 14,
                    "attempted_three_point_field_goals": 33,
                    "made_free_throws": 27,
                    "attempted_free_throws": 33,
                    "offensive_rebounds": 6,
                    "defensive_rebounds": 34,
                    "assists": 25,
                    "steals": 8,
                    "blocks": 8,
                    "turnovers": 12,
                    "personal_fouls": 20,
                    "points": 131,
                },
            ]
        )

    def test_2001_01_01_team_box_scores_with_charlotte_hornets(self):
        team_box_scores = client.team_box_scores(day=1, month=1, year=2001)
        self.assertListEqual(
            team_box_scores,
            [
                {
                    "team": Team.HOUSTON_ROCKETS,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 240,
                    "made_field_goals": 31,
                    "attempted_field_goals": 77,
                    "made_three_point_field_goals": 6,
                    "attempted_three_point_field_goals": 17,
                    "made_free_throws": 28,
                    "attempted_free_throws": 37,
                    "offensive_rebounds": 10,
                    "defensive_rebounds": 26,
                    "assists": 16,
                    "steals": 4,
                    "blocks": 3,
                    "turnovers": 11,
                    "personal_fouls": 19,
                    "points": 96,
                },
                {
                    "team": Team.MINNESOTA_TIMBERWOLVES,
                    "outcome": Outcome.WIN,
                    "minutes_played": 240,
                    "made_field_goals": 41,
                    "attempted_field_goals": 82,
                    "made_three_point_field_goals": 3,
                    "attempted_three_point_field_goals": 9,
                    "made_free_throws": 21,
                    "attempted_free_throws": 27,
                    "offensive_rebounds": 11,
                    "defensive_rebounds": 36,
                    "assists": 35,
                    "steals": 6,
                    "blocks": 2,
                    "turnovers": 8,
                    "personal_fouls": 29,
                    "points": 106,
                },
                {
                    "team": Team.CHARLOTTE_HORNETS,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 240,
                    "made_field_goals": 26,
                    "attempted_field_goals": 80,
                    "made_three_point_field_goals": 3,
                    "attempted_three_point_field_goals": 10,
                    "made_free_throws": 12,
                    "attempted_free_throws": 14,
                    "offensive_rebounds": 15,
                    "defensive_rebounds": 26,
                    "assists": 16,
                    "steals": 7,
                    "blocks": 1,
                    "turnovers": 15,
                    "personal_fouls": 25,
                    "points": 67,
                },
                {
                    "team": Team.PORTLAND_TRAIL_BLAZERS,
                    "outcome": Outcome.WIN,
                    "minutes_played": 240,
                    "made_field_goals": 29,
                    "attempted_field_goals": 70,
                    "made_three_point_field_goals": 4,
                    "attempted_three_point_field_goals": 15,
                    "made_free_throws": 27,
                    "attempted_free_throws": 30,
                    "offensive_rebounds": 11,
                    "defensive_rebounds": 36,
                    "assists": 20,
                    "steals": 8,
                    "blocks": 4,
                    "turnovers": 14,
                    "personal_fouls": 17,
                    "points": 89,
                }
            ])

    def test_2004_01_02_team_box_scores_with_new_orleans_hornets(self):
        team_box_scores = client.team_box_scores(day=11, month=12, year=2003)
        self.assertListEqual(
            team_box_scores,
            [
                {
                    "team": Team.DETROIT_PISTONS,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 240,
                    "made_field_goals": 28,
                    "attempted_field_goals": 78,
                    "made_three_point_field_goals": 3,
                    "attempted_three_point_field_goals": 8,
                    "made_free_throws": 27,
                    "attempted_free_throws": 32,
                    "offensive_rebounds": 10,
                    "defensive_rebounds": 18,
                    "assists": 19,
                    "steals": 12,
                    "blocks": 2,
                    "turnovers": 14,
                    "personal_fouls": 25,
                    "points": 86,
                },
                {
                    "team": Team.CLEVELAND_CAVALIERS,
                    "outcome": Outcome.WIN,
                    "minutes_played": 240,
                    "made_field_goals": 35,
                    "attempted_field_goals": 66,
                    "made_three_point_field_goals": 2,
                    "attempted_three_point_field_goals": 5,
                    "made_free_throws": 23,
                    "attempted_free_throws": 30,
                    "offensive_rebounds": 10,
                    "defensive_rebounds": 35,
                    "assists": 25,
                    "steals": 6,
                    "blocks": 9,
                    "turnovers": 22,
                    "personal_fouls": 30,
                    "points": 95,
                },
                {
                    "team": Team.SAN_ANTONIO_SPURS,
                    "outcome": Outcome.WIN,
                    "minutes_played": 240,
                    "made_field_goals": 27,
                    "attempted_field_goals": 84,
                    "made_three_point_field_goals": 6,
                    "attempted_three_point_field_goals": 21,
                    "made_free_throws": 11,
                    "attempted_free_throws": 21,
                    "offensive_rebounds": 15,
                    "defensive_rebounds": 31,
                    "assists": 17,
                    "steals": 11,
                    "blocks": 6,
                    "turnovers": 12,
                    "personal_fouls": 13,
                    "points": 71,
                },
                {
                    "team": Team.HOUSTON_ROCKETS,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 240,
                    "made_field_goals": 30,
                    "attempted_field_goals": 75,
                    "made_three_point_field_goals": 2,
                    "attempted_three_point_field_goals": 5,
                    "made_free_throws": 5,
                    "attempted_free_throws": 6,
                    "offensive_rebounds": 7,
                    "defensive_rebounds": 35,
                    "assists": 15,
                    "steals": 7,
                    "blocks": 8,
                    "turnovers": 20,
                    "personal_fouls": 24,
                    "points": 67,
                },
                {
                    "team": Team.NEW_ORLEANS_HORNETS,
                    "outcome": Outcome.WIN,
                    "minutes_played": 240,
                    "made_field_goals": 40,
                    "attempted_field_goals": 81,
                    "made_three_point_field_goals": 10,
                    "attempted_three_point_field_goals": 27,
                    "made_free_throws": 21,
                    "attempted_free_throws": 29,
                    "offensive_rebounds": 12,
                    "defensive_rebounds": 28,
                    "assists": 18,
                    "steals": 11,
                    "blocks": 3,
                    "turnovers": 16,
                    "personal_fouls": 17,
                    "points": 111,
                },
                {
                    "team": Team.PHOENIX_SUNS,
                    "outcome": Outcome.LOSS,
                    "minutes_played": 240,
                    "made_field_goals": 40,
                    "attempted_field_goals": 74,
                    "made_three_point_field_goals": 9,
                    "attempted_three_point_field_goals": 17,
                    "made_free_throws": 12,
                    "attempted_free_throws": 18,
                    "offensive_rebounds": 3,
                    "defensive_rebounds": 22,
                    "assists": 28,
                    "steals": 8,
                    "blocks": 1,
                    "turnovers": 13,
                    "personal_fouls": 23,
                    "points": 101,
                },
            ]
        )

    def test_2019_10_22_box_score_teams_and_outcomes(self):
        team_box_scores = client.team_box_scores(day=22, month=10, year=2019)
        lakers_box_score = team_box_scores[0]

        self.assertEqual(Team.LOS_ANGELES_LAKERS, lakers_box_score["team"])
        self.assertEqual(Outcome.LOSS, lakers_box_score["outcome"])
        self.assertEqual(102, lakers_box_score["points"])

        clippers_box_score = team_box_scores[1]

        self.assertEqual(Team.LOS_ANGELES_CLIPPERS, clippers_box_score["team"])
        self.assertEqual(Outcome.WIN, clippers_box_score["outcome"])
        self.assertEqual(112, clippers_box_score["points"])

        pelicans_box_score = team_box_scores[2]

        self.assertEqual(Team.NEW_ORLEANS_PELICANS, pelicans_box_score["team"])
        self.assertEqual(Outcome.LOSS, pelicans_box_score["outcome"])
        self.assertEqual(122, pelicans_box_score["points"])

        raptors_box_score = team_box_scores[3]

        self.assertEqual(Team.TORONTO_RAPTORS, raptors_box_score["team"])
        self.assertEqual(Outcome.WIN, raptors_box_score["outcome"])
        self.assertEqual(130, raptors_box_score["points"])


class TestTeamBoxScoresCSVOutput(TestCase):
    def setUp(self):
        self.file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/2018_01_01_team_box_scores.csv"
        )

    def tearDown(self):
        os.remove(self.file_path)

    def output_2018_01_01_team_box_scores_to_csv(self):
        client.team_box_scores(
            day=1,
            month=1,
            year=2018,
            output_type=OutputType.CSV,
            output_file_path=self.file_path,
            output_write_option=OutputWriteOption.WRITE
        )

    def test_2018_01_01_team_box_scores_csv_box_scores_to_file(self):
        self.output_2018_01_01_team_box_scores_to_csv()
        self.assertTrue(Path(self.file_path).exists())
        self.assertTrue(Path(self.file_path).is_file())

    def test_2019_01_01_team_box_scores_csv_values(self):
        self.output_2018_01_01_team_box_scores_to_csv()
        expected_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/2018_01_01_team_box_scores.csv"
        )
        with open(self.file_path, 'r') as output_file, open(expected_file_path, 'r') as expected_file:
            output_lines = output_file.readlines()
            expected_lines = expected_file.readlines()

        self.assertEqual(output_lines, expected_lines)


class TestTeamBoxScoresInMemoryJSON(TestCase):
    def setUp(self):
        self.expected_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/2018_01_01_team_box_scores.json"
        )

    def test_2018_01_01_team_box_scores_json_box_scores_to_memory(self):
        january_first_box_scores = client.team_box_scores(
            day=1,
            month=1,
            year=2018,
            output_type=OutputType.JSON,
        )
        with open(self.expected_file_path) as expected_json:
            self.assertEqual(json.loads(january_first_box_scores), json.load(expected_json))


class TestTeamBoxScoresJSONOutput(TestCase):
    def setUp(self):
        self.file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/2018_01_01_team_box_scores.json"
        )
        self.expected_file_path = os.path.join(
            os.path.dirname(__file__),
            "../output/expected/2018_01_01_team_box_scores.json"
        )

    def tearDown(self):
        os.remove(self.file_path)

    def test_2018_01_01_team_box_scores_json_box_scores_output_to_file(self):
        client.team_box_scores(
            day=1,
            month=1,
            year=2018,
            output_type=OutputType.JSON,
            output_file_path=self.file_path,
            output_write_option=OutputWriteOption.WRITE
        )
        self.assertTrue(Path(self.file_path).exists())
        self.assertTrue(Path(self.file_path).is_file())

    def test_2018_01_01_team_box_scores_json_box_scores_match_expected_output(self):
        client.team_box_scores(
            day=1,
            month=1,
            year=2018,
            output_type=OutputType.JSON,
            output_file_path=self.file_path,
            output_write_option=OutputWriteOption.WRITE
        )

        with open(self.file_path, 'r') as output_file, open(self.expected_file_path, 'r') as expected_file:
            self.assertEqual(
                json.load(output_file),
                json.load(expected_file),
            )



from unittest import TestCase
from unittest.mock import patch, PropertyMock, MagicMock

from basketball_reference_web_scraper.html import PlayByPlayRow


class TestPlayByPlayRow(TestCase):
    @patch.object(PlayByPlayRow, 'home_team_play_description', new_callable=PropertyMock)
    def test_is_home_team_play_when_home_team_play_description_is_not_empty_string(self, mocked_play_description):
        mocked_play_description.return_value = "jaebaebae"

        self.assertTrue(PlayByPlayRow(html=MagicMock()).is_home_team_play)

    @patch.object(PlayByPlayRow, 'home_team_play_description', new_callable=PropertyMock)
    def test_is_not_home_team_play_when_home_team_play_description_is_empty_string(self, mocked_play_description):
        mocked_play_description.return_value = ""

        self.assertFalse(PlayByPlayRow(html=MagicMock()).is_home_team_play)



import os
from datetime import datetime, timedelta
from unittest import TestCase

import pytz
import requests
from lxml import html

from basketball_reference_web_scraper.data import Team, TEAM_NAME_TO_TEAM
from basketball_reference_web_scraper.html import SchedulePage
from basketball_reference_web_scraper.parsers import ScheduledGamesParser, TeamNameParser, ScheduledStartTimeParser


class TestScheduleParser(TestCase):
    def setUp(self):
        self.october_2001_html = requests.get(
            'https://www.basketball-reference.com/leagues/NBA_2001_games.html'
        ).text
        self.october_2018_html = requests.get(
            'https://www.basketball-reference.com/leagues/NBA_2018_games-october.html'
        ).text
        self.schedule_with_future_games_html_file = open(
            os.path.join(os.path.dirname(__file__),
                         '../files/NBA_2019_games-april.html')
        )
        self.schedule_with_future_games_html = self.schedule_with_future_games_html_file.read()
        self.parser = ScheduledGamesParser(
            start_time_parser=ScheduledStartTimeParser(),
            team_name_parser=TeamNameParser(team_names_to_teams=TEAM_NAME_TO_TEAM),
        )

    def tearDown(self):
        self.schedule_with_future_games_html_file.close()

    def test_parse_october_2001_schedule_for_month_url_paths_(self):
        page = SchedulePage(html=html.fromstring(self.october_2001_html))
        expected_urls = [
            "/leagues/NBA_2001_games-november.html",
            "/leagues/NBA_2001_games-december.html",
            "/leagues/NBA_2001_games-january.html",
            "/leagues/NBA_2001_games-february.html",
            "/leagues/NBA_2001_games-march.html",
            "/leagues/NBA_2001_games-april.html",
            "/leagues/NBA_2001_games-may.html",
            "/leagues/NBA_2001_games-june.html",
        ]
        self.assertEqual(page.other_months_schedule_urls, expected_urls)

    def test_parse_october_2001_schedule(self):
        page = SchedulePage(html=html.fromstring(self.october_2001_html))
        parsed_schedule = self.parser.parse_games(games=page.rows)
        first_game = parsed_schedule[0]
        expected_datetime = pytz.timezone("US/Eastern") \
            .localize(datetime(year=2000, month=10, day=31, hour=19, minute=30)) \
            .astimezone(pytz.utc)

        self.assertEqual(len(parsed_schedule), 13)
        self.assertTrue(abs(first_game["start_time"] - expected_datetime) < timedelta(seconds=1))
        self.assertEqual(first_game["away_team"], Team.CHARLOTTE_HORNETS)
        self.assertEqual(first_game["home_team"], Team.ATLANTA_HAWKS)
        self.assertEqual(first_game["away_team_score"], 106)
        self.assertEqual(first_game["home_team_score"], 82)

    def test_parse_october_2018_schedule(self):
        page = SchedulePage(html=html.fromstring(self.october_2018_html))
        parsed_schedule = self.parser.parse_games(games=page.rows)
        self.assertEqual(len(parsed_schedule), 104)

    def test_parse_future_game(self):
        page = SchedulePage(html=html.fromstring(self.schedule_with_future_games_html))
        parsed_schedule = self.parser.parse_games(games=page.rows)
        first_game = parsed_schedule[0]
        expected_first_game_start_time = pytz.timezone("US/Eastern") \
            .localize(datetime(year=2019, month=4, day=1, hour=19, minute=30)) \
            .astimezone(pytz.utc)

        self.assertIsNotNone(parsed_schedule)
        self.assertEqual(len(parsed_schedule), 79)
        self.assertEqual(first_game["start_time"], expected_first_game_start_time)
        self.assertEqual(first_game["away_team"], Team.MIAMI_HEAT)
        self.assertEqual(first_game["home_team"], Team.BOSTON_CELTICS)
        self.assertIsNone(first_game["away_team_score"])
        self.assertIsNone(first_game["home_team_score"])



from unittest import TestCase
from unittest.mock import MagicMock, PropertyMock, patch

from basketball_reference_web_scraper.html import PlayerSeasonBoxScoresPage


class TestPlayerSeasonBoxScoresPage(TestCase):
    def setUp(self):
        self.html = MagicMock()

    def test_regular_season_box_scores_table_query(self):
        self.assertEqual(
            '//table[@id="pgl_basic"]',
            PlayerSeasonBoxScoresPage(html=self.html).regular_season_box_scores_table_query,
        )

    @patch.object(PlayerSeasonBoxScoresPage, "regular_season_box_scores_table_query", new_callable=PropertyMock)
    def test_regular_season_box_scores_table_is_none_when_no_matching_tables(self, mocked_query):
        mocked_query.return_value = "foobar"
        self.html.xpath = MagicMock(return_value=[])

        self.assertIsNone(PlayerSeasonBoxScoresPage(html=self.html).regular_season_box_scores_table)
        self.html.xpath.assert_called_once_with("foobar")

    @patch.object(PlayerSeasonBoxScoresPage, "regular_season_box_scores_table_query", new_callable=PropertyMock)
    def test_regular_season_box_scores_table_is_first_value_when_there_are_matching_tables(self, mocked_query):
        mocked_query.return_value = "foobar"
        first_value = MagicMock(name="First Matching Table")
        self.html.xpath = MagicMock(return_value=[first_value])
        table = PlayerSeasonBoxScoresPage(html=self.html).regular_season_box_scores_table

        self.assertIsNotNone(table)
        self.assertEqual(first_value, table.html)



from unittest import TestCase

from basketball_reference_web_scraper.data import Location, LOCATION_ABBREVIATIONS_TO_POSITION
from basketball_reference_web_scraper.parsers import LocationAbbreviationParser


class TestLocationAbbreviationParser(TestCase):
    def setUp(self):
        self.parser = LocationAbbreviationParser(abbreviations_to_locations=LOCATION_ABBREVIATIONS_TO_POSITION)

    def test_parse_away_symbol(self):
        self.assertEqual(Location.AWAY, self.parser.from_abbreviation("@"))

    def test_parse_home_symbol(self):
        self.assertEqual(Location.HOME, self.parser.from_abbreviation(""))

    def test_parse_unknown_location_symbol(self):
        self.assertRaises(ValueError, self.parser.from_abbreviation, "jaebaebae")